var _ = require('lodash');

var ConfigService = {
  getItems: function(groups) {
    return _.without(_.flatten(_.map(groups, function(group) {
      return _.map(_.get(group, 'items', []), function(item) {
        if (!_.isEmpty(item)) {
          return item;
        }
      });
    })), null);
  },

  getItem: function(groups, itemName) {
    var item = null;
    _.some(groups, function(group) {
      _.some(group.items, function(otherItem) {
        if (otherItem.name === itemName) {
          item = otherItem;
          return true;
        }
        return _.some(otherItem.items, function(otherChildItem) {
          if (otherChildItem.name === itemName) {
            item = otherChildItem;
            return true;
          }
          return false;
        });
      });
    });
    return item;
  },

  evaluateWhen: function(groups, when) {
    var checkWhen = function(obj, value, ne) {
      var val = _.get(obj, 'value', null);
      if (ne) {
        return val && val.length > 0 ? val !== value : obj.default !== value;
      }
      return val && val.length > 0 ? val === value : obj.default === value;
    };

    if (when.length > 0) {
      var whenParts = when.split('=', 2);
      if (whenParts.length === 2) {
        var opNe = false;
        var itemName = whenParts[0];
        var itemValue = whenParts[1];
        if (itemName.substr(itemName.length - 1) === '!') {
          opNe = true;
          itemName = itemName.substr(0, itemName.length - 1);
        }

        var theItem = this.getItem(groups, itemName);
        if (theItem !== null) {
          return checkWhen(theItem, itemValue, opNe);
        }
      }
    }

    return true;
  },

  isItemVisible: function(groups, item) {
    return !item.hidden && this.isEnabled(groups, item);
  },

  isEnabled: function(groups, obj) {
    var when = _.get(obj, 'when', '');
    return when.length > 0 ? this.evaluateWhen(groups, when) : true;
  },

  getEnabledGroups: function(groups) {
    return _(groups)
      .filter(function(group) {
        return !_.isEmpty(group);
      })
      .filter(this.isEnabled.bind(this, groups))
      .value();
  },
};

module.exports = ConfigService;
